unit UGame;

interface

uses windows,Classes, URender, UMath, UInput;

type
  //----------------------------------------------------------------------------
  // Player scene
  TPlayer = class(TScene)
    input:TInputKeyboard;
    pos:TVector;
    roll:Double;
    pitch:Double;
    shots:TList;
  public
    function Init:boolean;override;
    procedure Update( delta:Double );override;
  end;

  //----------------------------------------------------------------------------
  // Shot scene
  TShot = class(TScene)
    distance:Double;
    pos:TVector;
  public
    function Init:boolean;override;
    procedure Update( delta:Double );override;

    function  GetPos:TVector;
    procedure SetPos(a:TVector);
  end;


implementation

const
  //-------------------------------------------------
  shot_vtx:array[0..2]of TVector = (
    ( x: 0; y: 0; z: 4 ),
    ( x: 4; y: 0; z:-4 ),
    ( x:-4; y: 0; z:-4 )
  );
  shot_idx:array[0..2]of integer = (
    0,1,2
  );

  //-------------------------------------------------
  ship_vtx:array[0..3]of TVector = (
    ( x:  0; y:20; z: 0  ),
    ( x:-20; y: 0; z:-14 ),
    ( x: 20; y: 0; z:-14 ),
    ( x:  0; y: 0; z: 50 )
  );
  ship_idx:array[0..11]of integer = (
    0,1,2,
    0,1,3,
    0,2,3,
    1,2,3
  );
  //-------------------------------------------------


//----------------------------------------------------------------------------
function TPlayer.Init:boolean;
begin
  input := TInputKeyboard.Create;
  shots := TList.Create;
  pos   := VecInit(0,0,0);
  roll  := 0;
  pitch := 0;
  Result:=True;
end;

procedure TPlayer.Update( delta:Double );
var
  i:integer;
  m,view:TMatrix;
  shot:TShot;
begin

  //-------------------------------------------------
  //L[Xe[gE
  input.GetState;

  //-------------------------------------------------
  //e
  if input.IsPress( VK_SPACE ) then begin
    shot := TShot.Create(Renderer^);
    shot.SetPos(VecInit(pos.x,pos.y-pitch,pos.z+50));
    shots.Add(shot);
  end;

  //-------------------------------------------------
  //[
  if input.IsPress( VK_LEFT ) then begin
    if roll <  45 then roll:=roll+1;
    pos.x := pos.x-1;
  end else if input.IsPress( VK_RIGHT ) then begin
    if roll > -45 then roll:=roll-1;
    pos.x := pos.x+1;
  end else begin
    if roll > 0 then begin
      roll:=roll-1;
      if roll < 0 then roll:=0;
    end else if roll < 0 then begin
      roll:=roll+1;
      if roll > 0 then roll:=0;
    end;
  end;

  //-------------------------------------------------
  //sb`
  if input.IsPress( VK_DOWN ) then begin
    if pitch <  10 then pitch:=pitch+1;
    pos.y := pos.y-1;
  end else if input.IsPress( VK_UP ) then begin
    if pitch > -10 then pitch:=pitch-1;
    pos.y := pos.y+1;
  end else begin
    if pitch > 0 then begin
      pitch:=pitch-1;
      if pitch < 0 then pitch:=0;
    end else if pitch < 0 then begin
      pitch:=pitch+1;
      if pitch > 0 then pitch:=0;
    end;
  end;

  //-------------------------------------------------
  //eړ
  if shots.Count > 0 then
  begin
    i:=0;
    while i<shots.Count do
    begin
      shot := shots[i];
      if shot.GetPos.z > 1000 then begin
        shot.Free;
        shots.Delete(i);
      end else begin
        shot := shots[i];
        shot.Update(delta);
        inc(i);
      end;
    end;
  end;

  //-------------------------------------------------
  //_O
  m := MatMult( MatYawPitchRoll(0,Deg2Rad(pitch),Deg2Rad(roll)),
                MatTranslate(pos.x,pos.y,pos.z) );
  view := MatLookAtView( VecInit(0,100,-100), VecInit(pos.x,pos.y,0), VecInit(0,1,0) );
  Renderer.SetWorld(m);
  Renderer.SetView(view);
  Renderer.DrawIdxPrim( ship_vtx, ship_idx );

end;



//----------------------------------------------------------------------------
function TShot.Init:boolean;
begin
  pos    := VecInit(0,0,0);
  Result := True;
end;

procedure TShot.Update( delta:Double );
var m:TMatrix;
begin
  m := MatMult( MatYawPitchRoll(0,0,0), MatTranslate(pos.x,pos.y,pos.z) );
  Renderer.SetWorld(m);
  Renderer.DrawIdxPrim( shot_vtx, shot_idx );
  pos.z := pos.z + 10;
end;

procedure TShot.SetPos(a:TVector);
begin
  pos:=a;
end;

function TShot.GetPos:TVector;
begin
  Result:=pos;
end;









end.
